;; cups-daemon.scm -- printer spooler

(define cups (cat usr-sbin-dir "cupsd"))

(define cups-piddir "/var/run/cups")
(define cups-pidfile (cat cups-piddir "/cupsd.pid"))

(define cups-service
  (service '(cups)
    #:requirement '(rsyslogd)
    #:start (lambda args
	      (if (file-exists? cups)
		  (begin
		    (unless (file-exists? cups-piddir) (mkdir cups-piddir))
		    (unless (file-exists? (cat cups-piddir "/certs")) (mkdir (cat cups-piddir "/certs")))
		    ;; TODO: TZ and USB printers
		    (boot-format "[....] Starting cups...")
		    (let ((pid ((make-forkexec-constructor (list cups)))))
		      (if pid
			  (begin
			    (with-output-to-file cups-pidfile (lambda () (display (process-id pid)) (newline)))
			    (boot-format-ok)
			    #t)
			  (begin
			    (boot-format-fail)
			    #f))))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? cups-pidfile))
		   (boot-format "[....] Stopping cups...")
		   (let ((pid (with-input-from-file cups-pidfile read)))
		     (delete-file cups-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    ))

(register-services (list cups-service))
