;; ntpsec.scm -- ntpd server

(define ntpd (cat usr-sbin-dir "ntpd"))
(define ntp-pidfile "/var/run/ntpd.pid")

(define (ntpd-start)
  (lambda args
    (if (and (file-exists? ntpd) (catch #t (lambda () (getpwnam "ntpsec"))
					(lambda (keys . args) #f)))
	(let* ((uid (catch-system-error (passwd:uid (getpwnam "ntpsec"))))
	       (gid (catch-system-error (passwd:gid (getpwnam "ntpsec"))))
	       (runas (and uid gid (simple-format #f "~a:~a" uid gid))))
	  (boot-format "[....] Starting ntpd daemon...")
	  ;; TODO handle /run/ntp.conf.dhcp
	  ;; notice that ntpd forks so #:pid-file is not usable
	  (if (and runas ((make-forkexec-constructor (list ntpd "-g" "-p" ntp-pidfile "-u" runas))))
	      (begin
		(boot-format-ok)
		#t)
	      (begin
		(boot-format-fail)
		#f)
	      ))
	#f)
    ))

(define ntp-service
  (service '(ntpsec)
    #:requirement '(single-user)
    #:start (ntpd-start)
    #:stop (lambda (running)
	     (when (and running (file-exists? ntp-pidfile))
		   (boot-format "[....] Stopping ntpd daemon...")
		   (let ((pid (with-input-from-file ntp-pidfile read)))
		     (delete-file ntp-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    ))

(register-services (list ntp-service))
