;; php-fpm.scm -- fastcgi pour php

(define php-fpm-piddir "/run/php")

;; we need to find out version at run time
(define (fpm-version) (let ((ret (system-value "dpkg-query --showformat='${Version}\n' --show php-fpm| cut -d':' -f2| cut -d'+' -f1")))
		      (if (eof-object? ret)
			  ""
			  ret)))

(define (php-fpm) (cat usr-sbin-dir "php-fpm" (fpm-version)))
(define (conf-file) (cat "/etc/php/" (fpm-version) "/fpm/php-fpm.conf"))

;; should be read from configuration file
(define (php-fpm-pidfile) (cat php-fpm-piddir "/php" (fpm-version) "-fpm.pid"))

(define (create-dir-php dir uid gid)
  "uid and gid can be integers or strings"
  (unless (and (file-exists? dir)
	       (eq? (stat:type (lstat dir)) 'directory))
    (if (file-exists? dir)
	(delete-file dir)
	(mkdir dir)))
  (chmod dir #o750)
  (chown dir
	 (if (string? uid)
	     (let ((r (catch-system-error (passwd:uid (getpwnam uid)))))
	       (if r r 0))
	     uid)
	 (if (string? gid)
	     (let ((r (catch-system-error (group:gid (getgrnam gid)))))
	       (if r r 0))
	     gid)
	 ))

(define php-fpm-service
  (service '(php-fpm)
    #:requirement '(apache)
    #:start (lambda args
	      (if (file-exists? (php-fpm))
		  (begin
		    (boot-format (cat "[....] Starting php-fpm" (fpm-version) "..."))
		    (create-dir-php php-fpm-piddir "www-data" "www-data")
		    (if ((make-forkexec-constructor (list (php-fpm) "--fpm-config" (conf-file))))
			(begin
			  (boot-format-ok)
			  #t)
			(begin
			  (boot-format-fail)
			  #f)
			))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? (php-fpm-pidfile)))
	       (boot-format (cat "[....] Stopping php-fpm" (fpm-version) "..."))
	       (let ((pid (with-input-from-file (php-fpm-pidfile) read))
		     (proc (cat "pgrep -c -d ' ' php-fpm" (fpm-version))))
		 (catch-system-error (kill pid SIGQUIT))
		 ;; children can take a long time to exit
		 (let ((ret (do ((i 1 (1+ i)))
				((or (<= (system-value-read proc) 1)
				     (> i 5))
				 (if (> i 5)
				     #f
				     #t))
			      (sleep 1)
			      (boot-format "*"))))
		   (if ret
		       (boot-format-ok)
		       (begin
			 (catch-system-error (kill pid SIGTERM))
			 (let ((ret (do ((i 1 (1+ i)))
					((or (<= (system-value-read proc) 0)
					     (> i 5))
					 (if (> i 5)
					     (catch-system-error (kill pid SIGKILL))
					     #t))
				      (sleep 1)
				      (boot-format "*"))))
			   (if ret
			       (begin
				 (when (file-exists? (php-fpm-pidfile))
				   (delete-file (php-fpm-pidfile)))
				 (boot-format-ok))
			       (boot-format-fail))))))))
	     #f)
    ))

(register-services (list php-fpm-service))
