;; unbound.scm -- unbound recursive DNS server

(define unbound (cat usr-sbin-dir "unbound"))

(define unbound-pidfile "/var/run/unbound.pid")

(define unbound-service
  (service '(unbound)
    #:requirement '(fullnetwork)
    #:start (lambda args
	      (if (file-exists? unbound)
		  (begin
		    (boot-format "[....] Starting unbound...")
		    (let ((ret (status:exit-val (system-pipe "/usr/sbin/unbound-anchor -a /var/lib/unbound/root.key -v"
							     (lambda (x) (boot-format "~a..." x))))))
		      (if (zero? ret)
			  (boot-format "root anchor updated...")
			  (boot-format "root anchor unchanged...")))
		    (let* ((uid (catch-system-error (passwd:uid (getpwnam "unbound"))))
			   (gid (catch-system-error (group:gid (getgrnam "unbound")))))
		      (when (and uid gid) (chown "/var/lib/unbound/root.key" uid gid)))
		    (let ((pid ((make-forkexec-constructor (list unbound)))))
		      (if pid
			  (begin
			    (with-output-to-file unbound-pidfile (lambda () (display (process-id pid)) (newline)))
			    (boot-format-ok)
			    #t)
			  (begin
			    (boot-format-fail)
			    #f))))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? unbound-pidfile))
		   (boot-format "[....] Stopping unbound...")
		   (let ((pid (with-input-from-file unbound-pidfile read)))
		     (delete-file unbound-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    )
  )

(register-services (list unbound-service))


