;; uuid-runtime.scm -- uuidd daemon

(define uuidd (cat usr-sbin-dir "uuidd"))
(define uuid-dirfile "/run/uuidd")
(define uuid-pidfile (cat uuid-dirfile "/uuidd.pid"))

(define uuid-service
  (service '(uuidd)
    #:requirement '(single-user)
    #:start (lambda args
	      (if (file-exists? uuidd)
		  (begin
		    (boot-format "[....] Starting uuidd daemon...")
		    (unless (file-exists? uuid-dirfile)
			    (mkdir uuid-dirfile))
		    (let* ((uid (catch-system-error (passwd:uid (getpwnam "uuidd"))))
			   (gid (catch-system-error (group:gid (getgrnam "uuidd")))))
		      (if (and uid gid)
			  (begin
			    (chown uuid-dirfile uid gid)
			    (let ((pid ((make-forkexec-constructor (list uuidd) #:pid-file uuid-pidfile))))
			      (if pid
				  (begin
				    (boot-format-ok)
				    #t)
				  (begin
				    (boot-format-fail)
				    #f)
				  )))
			  (begin
			    (boot-format-fail)
			    #f))))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? uuid-pidfile))
		   (boot-format "[....] Stopping uuidd daemon...")
		   (let ((pid (with-input-from-file uuid-pidfile read)))
		     (delete-file uuid-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    ))

(register-services (list uuid-service))
